﻿'use strict';

var chatSession = {};

$(function () {
    window.addEventListener('ChatOpenMediaWindow', function (e) {
        if (window.location === window.parent.location)
            openMediaWindow(e.detail);
        else {
            e.detail.openMediaWindow = true;
            postData(e.detail);
        }
    }, false);

    function openMediaWindow(callParams) {
        var w = 740, h = 560;
        var win = window.parent;

        // Fixes dual-screen position                         Most browsers      Firefox
        var dualScreenLeft = win.screenLeft !== undefined ? win.screenLeft : win.screenX;
        var dualScreenTop = win.screenTop !== undefined ? win.screenTop : win.screenY;

        var width = win.innerWidth ? win.innerWidth : document.documentElement.clientWidth ? document.documentElement.clientWidth : screen.width;
        var height = win.innerHeight ? win.innerHeight : document.documentElement.clientHeight ? document.documentElement.clientHeight : screen.height;

        var left = width / 2 - w / 2 + dualScreenLeft;
        var top = height / 2 - h / 2 + dualScreenTop;

        var mediaWindow = window.open(buildChatMediaSessionUrl(callParams),
            callParams.sessionId, 'toolbar=no,width=' + w + ',height=' + h + ', top=' + top + ', left=' + left);

        if (window.focus)
            mediaWindow.focus();

        function buildChatMediaSessionUrl(params) {
            var baseUrl = window.location.protocol + "//" + window.location.host + RQ_URL + "/";
            return baseUrl + 'Chat/ChatMediaSession/?' + $.param(params);
        }
    }

    function subscribeViewModel(newViewModel) {
        window.addEventListener('message', function (e) {
            if (e.data.isObserved === true && e.data.chatProfileKey === newViewModel.chatProfileKey) {
                newViewModel.addWelcomeMessageIfNeeded();
                $("[data-profilekey='" + newViewModel.chatProfileKey + "'] textarea.chat-messages-send-textarea").focus();
            }
            else if (typeof e.data.highlighterClosed === 'boolean' && e.data.chatProfileKey === newViewModel.chatProfileKey) {
                chatUtils.closeHighlighter(newViewModel.chatProfileKey);
            }
        }, false);

        newViewModel.session.subscribe(function (newSession) {
           if (newSession.Form) {
               if ($("#tabForm_" + newViewModel.sessionId).hasClass("active")) {
                    newViewModel.addWelcomeMessageIfNeeded();
                    $('.list-group a[href="#tabChatMessages_' + newViewModel.sessionId + '"]').tab('show');
                    $('#msgInput_' + newViewModel.sessionId).focus();
                }
            }
           else if (needToFillForm()) {
               if ($("#tabChatMessages_" + newViewModel.sessionId).hasClass("active")) {
                    $('.list-group a[href="#tabForm_' + newViewModel.sessionId + '"]').tab('show');
                }
            }
        });

        newViewModel.callOptions.subscribe(function (newCallOptions) {
            if (newCallOptions && newCallOptions.isRinging)
            {
                $('.list-group a[href="#tabChatMessages_' + newViewModel.sessionId + '"]').tab('show');
                let $widget = $('#internalChatWidget_' + newViewModel.chatProfileKey);
                $('.chat-button-container', $widget).hide();
                $('.chat-highlighter-container', $widget).hide();
                $('.chat-all', $widget).show();
            }
        });

        newViewModel.messages.subscribe(function (newMessages) {
            if (newMessages.length > 0 && !needToFillForm()) {
                setTimeout(function () {
                    $('.list-group a[href="#tabChatMessages_' + newViewModel.sessionId + '"]').tab('show');
                    let $widget = $('#internalChatWidget_' + newViewModel.chatProfileKey);
                    if ($widget.length > 0) {
                        if ($('.chat-all', $widget).css('display') === 'none') {
                            if (document["hidden"]) {
                                document.title = Localization_Chat_NewChat + " - " + originalDocumentTitle;
                                var soundElement = document.getElementById("chatHostSound");
                                if (soundElement !== null) {
                                    if (!soundElement.paused)
                                        soundElement.currentTime = 0;

                                    soundElement.play();
                                }
                            }
                        }

                        $('.chat-button-container', $widget).hide();
                        $('.chat-highlighter-container', $widget).hide();
                        $('.chat-all', $widget).show();
                    }
                });
            }
        });

        newViewModel.offlineFormSent.subscribe(function (newOfflineFormSent) {
            if (newOfflineFormSent === true) {
                $('a[href="#tabFilledOfflineForm_' + newViewModel.sessionId + '"]').tab('show');
            }
            else if (newOfflineFormSent === false)
                $('a[href="#tabOfflineForm_' + newViewModel.sessionId + '"]').tab('show');
        });

        function needToFillForm() {
            var session = newViewModel.session();
            if (session) {
                var formIsActive = newViewModel.form() !== null && newViewModel.form() !== undefined;
                var formIsFilled = session.Form !== null && session.Form !== undefined;
                var firstMessage = newViewModel.messages().length > 0 ? newViewModel.messages()[0] : null;
                var firstMessageFromOperator = firstMessage && firstMessage.FromOperator && firstMessage.UserProviderKey.length > 0;
                return formIsActive && !formIsFilled && !firstMessageFromOperator &&
                    (session.ChatBotMode !== true || !newViewModel.chatProfile.ChatBot || !newViewModel.chatProfile.ChatBot.UseChatBotFormFlow);
            }
            else {
                return false;
            }
        }

        chatUtils.addVisibilityChangeListener(function () {
            document.title = originalDocumentTitle;
            newViewModel.visibleHandler();
        }, function () {
            newViewModel.notVisibleHandler();
        });
    }

    function initCustomBindings() {
        ko.bindingHandlers.automationHtml = {
            init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                function processAutomationText(text) {
                    return text ? text.replace(/{{OFFLINE_FORM_LINK,( )?".*"}}/i, function (match, p1, offset, string) {
                        let replacementText = match.substring(match.indexOf('"') + 1, match.lastIndexOf('"'));
                        if (bindingContext.$parent.offlineFormEnabled && bindingContext.$parent.myUserSession() === bindingContext.$parent.guestUserSession())
                            return '<a class="link" style="cursor: pointer; color: #007bff;" data-bind="click: function() { $parent.leaveSession().then(chatSession.showOfflineFormTab); }">' + replacementText + '</a>';

                        return '<a class="link" style="color: #007bff;">' + replacementText + '</a>';
                    }) : "";
                }

                element.innerHTML = processAutomationText(valueAccessor());
            }
        };

        ko.bindingHandlers.answerClick = {
            init: function (element, valueAccessor, allBindings, viewModel) {
                var params = valueAccessor();
                element.addEventListener("click", function (e) {
                    var inputElement = document.getElementById(params.inputElementId);
                    var startOfReplacement = inputElement.selectionStart;
                    var replacedText = inputElement.value.replaceAt(startOfReplacement, inputElement.selectionEnd - inputElement.selectionStart, params.text + " ");
                    var valueBinding = params.inputValue;
                    if (valueBinding)
                        valueBinding(replacedText);
                    else
                        inputElement.value = replacedText;

                    inputElement.selectionStart = inputElement.selectionEnd = startOfReplacement + params.text.length + 1;
                    inputElement.focus();
                });
            }
        };

        ko.bindingHandlers.hashtag = {
            init: function (element, valueAccessor, allBindings, viewModel) {
                var HASHTAG_MAX_LENGTH = 20;
                var SELECTED_HASHTAG_COLOR = "#A9A9A9";

                var $helper = $('<ul></ul>', {
                    "class": "dropdown-menu dropdown-helper",
                    role: "menu"
                });

                element.addEventListener("blur", function (e) {
                    setTimeout(function () {
                        $helper[0].style.display = "none";
                    }, 100);
                });

                $helper.appendTo(element.parentElement);

                function replaceHashtagByItsText(element, text) {
                    var deltaCounter = 0;
                    while (deltaCounter < HASHTAG_MAX_LENGTH + 1) {
                        let deltaChar = element.value.charAt(element.selectionStart - deltaCounter - 1);
                        if (deltaChar === '#') {
                            var startOfReplacement = element.selectionStart - deltaCounter - 1;
                            var replacedText = element.value.replaceAt(startOfReplacement, deltaCounter + 1, text + " ");
                            var textInputBinding = allBindings.get('textInput');
                            if (textInputBinding)
                                textInputBinding(replacedText);
                            else
                                element.value = replacedText;

                            $helper[0].style.display = "none";

                            break;
                        }
                        else {
                            if (deltaChar >= 'a' && deltaChar <= 'z' || deltaChar >= 'A' && deltaChar <= 'Z' || deltaChar >= '0' && deltaChar <= '9' || deltaChar === '-') {
                                ++deltaCounter;
                            }
                            else {
                                $helper[0].style.display = "none";
                                break;
                            }
                        }
                    }
                }

                function processSpecialKeyDownIfNeeded(e, getText) {
                    if ((e.which === 38 || e.which === 40 || e.which === 13 || e.which === 27) && e.type === "keydown" && $(".dropdown-helper", e.target.parentElement).is(":visible"))
                    {
                        switch (e.which) {
                            case 38: // arrow up
                                {
                                    let $lastSelected = $("li[data-selected='true']", $helper[0]);
                                    let $currentSelected = $lastSelected.prev();
                                    if ($currentSelected.length === 1) {
                                        $lastSelected.attr("data-selected", "false");
                                        $lastSelected.css("background-color", "initial");
                                        $currentSelected.attr("data-selected", "true");
                                        $currentSelected.css("background-color", SELECTED_HASHTAG_COLOR);
                                    }

                                    e.preventDefault();
                                    break;
                                }
                            case 40: // arrow down
                                {
                                    let $lastSelected = $("li[data-selected='true']", $helper[0]);
                                    let $currentSelected = $lastSelected.next();
                                    if ($currentSelected.length === 1) {
                                        $lastSelected.attr("data-selected", "false");
                                        $lastSelected.css("background-color", "initial");
                                        $currentSelected.attr("data-selected", "true");
                                        $currentSelected.css("background-color", SELECTED_HASHTAG_COLOR);
                                    }

                                    e.preventDefault();
                                    break;
                                }
                            case 13: // enter
                                let $selected = $("li[data-selected='true']", $helper[0]);
                                if ($selected.length === 1) {
                                    replaceHashtagByItsText(e.target, getText($('a:first', $selected[0]).text()));
                                }

                                e.preventDefault();
                                break;
                            case 27: // escape
                                $helper[0].style.display = "none";
                                e.preventDefault();
                                break;
                        }
                    }

                    return e.isDefaultPrevented();
                }

                function processSpecialKeyUpfNeeded(e) {
                    if (e.type === "keyup")
                    {
                        if (e.which === 27) // escape
                            e.preventDefault();
                        else if ((e.which === 38 || e.which === 40 || e.which === 13) && $(".dropdown-helper", e.target.parentElement).is(":visible")) {
                            e.preventDefault();
                        }
                    }

                    return e.isDefaultPrevented();
                }

                var params = valueAccessor();
                $(element).on('keydown keyup paste cut click', function (e) {
                    if (!processSpecialKeyDownIfNeeded(e, params.executeOnGetText) && e.type !== "keydown" && !processSpecialKeyUpfNeeded(e) && element.selectionStart == element.selectionEnd)
                    {
                        var deltaCounter = 0, hashtag = "";
                        while (true) {
                            let deltaChar = element.value.charAt(element.selectionStart - deltaCounter - 1);
                            if (deltaChar === '#') {
                                var elementClientRect = element.getBoundingClientRect();
                                element.selectionStart = element.selectionEnd = element.selectionStart - deltaCounter;
                                var caretPos = $(this).textareaHelper('caretPos');
                                element.selectionStart = element.selectionEnd = element.selectionStart + deltaCounter;

                                $helper[0].innerHTML = "";
                                var filteredHashtags = params.executeOnFilter.call(viewModel, hashtag);
                                if (filteredHashtags && filteredHashtags.length > 0) {
                                    $(filteredHashtags).each(function (index, item) {
                                        $('<li></li>', {
                                            html: $('<a></a>', {
                                                href: '#',
                                                text: item.Shortcut
                                            }),
                                            css: index === 0 ? { "background-color": SELECTED_HASHTAG_COLOR } : {},
                                            "data-selected": index === 0 ? true : false,
                                            click: function (e) {
                                                replaceHashtagByItsText(element, params.executeOnGetText(e.target.innerText));
                                            }
                                        }).appendTo($helper[0]);
                                    });
                                    
                                    $helper[0].style.display = "block";
                                    $helper[0].style.top = Math.round(elementClientRect.top + caretPos.top + document.documentElement.scrollTop - $helper[0].offsetHeight) + "px";
                                    $helper[0].style.left = Math.round(elementClientRect.left + caretPos.left + document.documentElement.scrollLeft) + "px";
                                }
                                else
                                    $helper[0].style.display = "none";

                                break;
                            }
                            else {
                                if (deltaCounter < HASHTAG_MAX_LENGTH && (deltaChar >= 'a' && deltaChar <= 'z' || deltaChar >= 'A' && deltaChar <= 'Z' || deltaChar >= '0' && deltaChar <= '9' || deltaChar === '-')) {
                                    hashtag = deltaChar + hashtag;
                                    ++deltaCounter;
                                }
                                else {
                                    $helper[0].style.display = "none";
                                    break;
                                }
                            }
                        }
                    }

                    return true;
                });

                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    $(element).textareaHelper('destroy');
                });
            }
        };
    }

    chatSession.isRightNotLeftMessage = function (message, viewModel) {
        switch (message.Type) {
            case chatUtils.ChatMessageTypeEnum.COMMON_INCOMPLETE:
            case chatUtils.ChatMessageTypeEnum.COMMON:
            case chatUtils.ChatMessageTypeEnum.ATTACHMENTS:
                return message.UserProviderKey === viewModel.userProviderKey;
            case chatUtils.ChatMessageTypeEnum.AUTOMATION:
            case chatUtils.ChatMessageTypeEnum.BOT_COMMON:
            case chatUtils.ChatMessageTypeEnum.BOT_OPTIONS:
                return false;
        }
    };

    chatSession.spots2Text = function (spots) {
        if (!spots)
            return null;

        return spots.map(function (spot) {
            return "{0} ({1})".format(spot.fullName, moment.utc(spot.timestamp).local().format(chatUtils.isToday(spot.timestamp) ? 'LT' : 'l LT'));
        }).join(", ");
    };

    chatSession.getMyAttitudeText = function (session, viewModel) {
        var myExpression = viewModel.getMyExpression(session);
        var isAcceptedByAnybody = viewModel.isAcceptedByAnybody(session);

        if (myExpression) {
            if (myExpression.Offer && myExpression.Attitude !== true) {
                return !isAcceptedByAnybody ? Localization_Chat_Accept : myExpression.Offer.Type === chatUtils.ChatSessionOfferTypeEnum.PassOn ? Localization_Chat_Take : Localization_Chat_Accept;
            }

            return myExpression.Attitude === true ? undefined : isAcceptedByAnybody ? undefined : session.WaitingMessage ? Localization_Chat_Accept : Localization_Chat_StartChat;
        }
        else
            return isAcceptedByAnybody ? undefined : session.WaitingMessage ? Localization_Chat_Accept : Localization_Chat_StartChat;

    };

    function postData(data, noOriginRestriction) {
        var origin = noOriginRestriction ? "*" : window.location !== window.parent.location ? document.referrer : document.location.href;

        if (window.parent)
            window.parent.postMessage(data, origin);

        if (window.opener)
            window.opener.postMessage(data, origin);
    }

    function initDOM() {
        $(".chat-all-header").click(function (e) {
            var chatProfileKey = $(this).closest('[data-profilekey]').attr('data-profilekey');
            postData({ minimize: true, chatProfileKey: chatProfileKey });
        });

        $(".guest-header-toolbox-close").click(function (e) {
            e.stopPropagation();
            $('[href^="#tabTermination_"]', $(this).closest('.chat-all')).tab('show');
        });
    }

    //ready
    var originalDocumentTitle = document.title;
    initCustomBindings();
    initDOM();
    chatSession.subscribeViewModel = subscribeViewModel;
    chatSession.showChatMessagesTab = function (sessionId) {
        $('.list-group a[href="#tabChatMessages_' + sessionId + '"]').tab('show');
    };
    chatSession.showChatTab = function (sessionId) {
        $('.list-group a[href="#tabChat_' + sessionId + '"]').tab('show');
    };
    chatSession.showOfflineFormTab = function (sessionId) {
        if (sessionId)
            $('.list-group a[href="#tabOfflineForm_' + sessionId + '"]').tab('show');
    };
    chatSession.setChatBotModeAndShowMessages = function (chatBotMode) {
        var self = this;
        self.setChatBotMode(chatBotMode).then(function () {
            $('.list-group a[href="#tabChatMessages_' + self.sessionId + '"]').tab('show');
            $('#msgInput_' + self.sessionId).focus();
        });
    };
});